SystemOrganization addCategory: #Actors!
SystemOrganization addCategory: #'Actors-Tests'!

TestCase subclass: #ActorTests
	instanceVariableNames: 'actor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actors-Tests'!

!ActorTests methodsFor: 'as yet unclassified' stamp: 'bs 2/22/2010 08:21:52.274'!
actorPort
	^ 10000! !

!ActorTests methodsFor: 'running' stamp: 'bs 2/22/2010 08:22:07.041'!
setUp
	actor := TestActor new.
	actor startOn: self actorPort.! !

!ActorTests methodsFor: 'running' stamp: 'bs 2/22/2010 08:14:54.551'!
tearDown
	actor stop.! !

!ActorTests methodsFor: 'as yet unclassified' stamp: 'bs 2/22/2010 08:24:16.792'!
testActorMessaging
	| testActorId aTextMessage |
	aTextMessage := 'Hello World'.
	testActorId := ActorId localhostPort: self actorPort.
	testActorId remember: aTextMessage.
	(Delay forMilliseconds: 100) wait.
	self should: [ actor receivedMessage = aTextMessage ].
	testActorId close.! !

Object subclass: #Actor
	instanceVariableNames: 'listenSocket servingProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actors'!

!Actor commentStamp: 'bs 2/22/2010 08:58:10.193' prior: 0!
An actor represents a process that can receive messages.
- This simple implementation uses standard Smalltalk processes, thus, it is not very economic compared to much better actor implementations.
- On startUp it listens on a socket for incoming messsages.
- An actor can receive all message the normal object can handle. However, in contrast to normal object message send, an actor client does not expect an response. Actor like messaging is only one-way communication

Sources:
- Erlang concurrency
- Actor programming model!

!Actor methodsFor: 'as yet unclassified' stamp: 'bs 2/21/2010 21:28:32.845'!
canProcess: aMessage
	^ (aMessage isKindOf: Message) and: [ self respondsTo: aMessage selector ]
	! !

!Actor methodsFor: 'as yet unclassified' stamp: 'bs 2/21/2010 21:09:52.631'!
messageEndMark
	^ 'ActorsEOF'! !

!Actor methodsFor: 'as yet unclassified' stamp: 'bs 2/21/2010 21:29:14.897'!
process: aSocket
	| socketStream messageBytes message |
	socketStream := SocketStream on: aSocket.
	messageBytes := socketStream upToAll: self messageEndMark.
	message := ReferenceStream unStream: messageBytes.
	(self canProcess: message) 
		ifTrue: [ message sendTo: self ]
	! !

!Actor methodsFor: 'as yet unclassified' stamp: 'bs 2/21/2010 21:22:28.952'!
runLoop
	"Run the server's connection accept loop."
	| newConnectionSocket |
	
	[listenSocket isValid]
		whileTrue: 
			[[newConnectionSocket := listenSocket waitForAcceptFor: 10]
				on: ConnectionTimedOut
				do: [:ex | newConnectionSocket := nil].
			 newConnectionSocket isNil
				ifFalse: [ self serve: newConnectionSocket ] ]! !

!Actor methodsFor: 'as yet unclassified' stamp: 'bs 2/21/2010 21:05:19.558'!
serve: aSocket
	[self process: aSocket]
		forkAt: Processor lowIOPriority
	! !

!Actor methodsFor: 'as yet unclassified' stamp: 'bs 2/21/2010 20:12:45.289'!
servicePriority
	^ Processor lowIOPriority! !

!Actor methodsFor: 'as yet unclassified' stamp: 'bs 2/21/2010 20:12:30.526'!
startOn: portNumber
	
	listenSocket := Socket newTCP.
	listenSocket listenOn: portNumber backlogSize: 2.
	listenSocket isValid ifFalse: [^self error: 'Could not create server socket'].
	servingProcess := [self runLoop] forkAt: self servicePriority.! !

!Actor methodsFor: 'as yet unclassified' stamp: 'bs 2/21/2010 20:13:14.621'!
stop
	servingProcess terminate.
	listenSocket closeAndDestroy.! !

Actor subclass: #TestActor
	instanceVariableNames: 'receivedMessage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actors-Tests'!

!TestActor methodsFor: 'accessing' stamp: 'bs 2/22/2010 08:11:13.701'!
receivedMessage
	^ receivedMessage! !

!TestActor methodsFor: 'accessing' stamp: 'bs 2/22/2010 08:11:13.703'!
receivedMessage: anObject
	receivedMessage := anObject! !

!TestActor methodsFor: 'as yet unclassified' stamp: 'bs 2/22/2010 08:12:25.272'!
remember: aMessage
	self receivedMessage: aMessage.! !

Object subclass: #ActorId
	instanceVariableNames: 'hostname port socketStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actors'!

!ActorId class methodsFor: 'as yet unclassified' stamp: 'bs 2/21/2010 21:35:34.52'!
hostname: aName port: anInteger
	^ self new
		hostname: aName port: anInteger;
		yourself! !

!ActorId class methodsFor: 'as yet unclassified' stamp: 'bs 2/21/2010 21:36:32.49'!
localhostPort: anInteger
	^ self new
		hostname: '127.0.0.1' port: anInteger;
		yourself! !

!ActorId methodsFor: 'as yet unclassified' stamp: 'bs 2/22/2010 08:18:39.63'!
close
	self socketStream close.! !

!ActorId methodsFor: 'as yet unclassified' stamp: 'bs 2/21/2010 21:58:47.124'!
doesNotUnderstand: aMessage
	self socketStream 
		nextPutAll: (ReferenceStream streamedRepresentationOf: aMessage);
		nextPutAll: self messageEndMark;
		flush! !

!ActorId methodsFor: 'as yet unclassified' stamp: 'bs 2/21/2010 21:35:11.327'!
hostname: aName port: anInteger
	hostname := aName.
	port := anInteger.! !

!ActorId methodsFor: 'as yet unclassified' stamp: 'bs 2/21/2010 21:53:45.9'!
messageEndMark
	^ 'ActorsEOF'! !

!ActorId methodsFor: 'as yet unclassified' stamp: 'bs 2/21/2010 21:33:30.694'!
port
	^ port! !

!ActorId methodsFor: 'as yet unclassified' stamp: 'bs 2/21/2010 21:58:18.33'!
socketStream
	| socket | 
	socketStream 
		ifNil: [
			socket := Socket newTCP.
			socket
				connectTo: (NetNameResolver addressFromString: '127.0.0.1') 
				port: self port.
			socketStream := SocketStream on: socket ].
	^ socketStream! !
